using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Imaging;

using Syncfusion.Diagnostics;
using Syncfusion.Windows.Forms.Grid;

namespace AdvancedTabbedMDI
{
    public class FKCellButton : GridCellButton
    {
        static GridIconPaint iconPainter;

        static FKCellButton()
        {
            iconPainter = new GridIconPaint("AdvancedTabbedMDI.", typeof(FKCellButton).Assembly);
        }

        /// <summary>
        /// Initializes a <see cref="FKCellButton"/> and associates it with a <see cref="GridTextBoxCellRenderer"/>
        /// </summary>
        /// <param name="control">The <see cref="GridTextBoxCellRenderer"/> that draws this cell button element.</param>
        public FKCellButton(GridCellRendererBase control)
            : base(control)
        {
        }

        /// <override/>
        public void Draw(Graphics g, int rowIndex, int colIndex, bool bActive, GridStyleInfo style, string fp_sBitmap)
        {
            TraceUtil.TraceCurrentMethodInfo(rowIndex, colIndex, bActive, style.CellValue);

            base.Draw(g, rowIndex, colIndex, bActive, style);

            // draw the button
            bool hovering = IsHovering(rowIndex, colIndex);
            bool mouseDown = IsMouseDown(rowIndex, colIndex);
            bool disabled = !style.Clickable;

            ButtonState buttonState = ButtonState.Normal;
            if (disabled)
                buttonState |= ButtonState.Inactive | ButtonState.Flat;

            else if (!hovering && !mouseDown)
                buttonState |= ButtonState.Flat;

            Point ptOffset = Point.Empty;
            if (mouseDown)
            {
                ptOffset = new Point(1, 1);
                buttonState |= ButtonState.Pushed;
            }

            if (bActive || style.ShowButtons != GridShowButtons.ShowCurrentCellEditing)
                DrawButton(g, Bounds, buttonState, style);

            // Instead of using GridIconPaint you could simple use Image.Draw here
            // with an existing bitmap. GridIconPaint is convenient because it lets
            // us easily draw over existing background and replace the black color
            // in the bitmap with a different color.
            iconPainter.PaintIcon(g, Bounds, ptOffset, fp_sBitmap, Color.Blue);
        }
    }

    public class FKCellButtonNewProperties : FKCellButton
    {
        public FKCellButtonNewProperties(GridCellRendererBase control)
            : base(control)
        {
        }

        public override void Draw(Graphics g, int rowIndex, int colIndex, bool bActive, GridStyleInfo style)
        {
            base.Draw(g, rowIndex, colIndex, bActive, style, "NewSmall.gif");
        }
    }

    /// Implements the data/model part for a textbox cell with ellipse in a <see cref="GridDataBoundGrid"/>. The
    /// The exandable row header cell will display a '+' for expanded rows and a '-' for collapsed rows similar to a TreeControl.
    /// a pencil when the row is being edited.
    /// </summary>
    [Serializable]
    public class FKCellModelSelectNew : GridTextBoxCellModel
    {
        /// <overload>
        /// Initializes a new <see cref="FKCellModelSelectNew"/> object.
        /// </overload>
        /// <summary>
        /// Initializes a new <see cref="FKCellModelSelectNew"/> from a serialization stream.
        /// </summary>
        /// <param name="info">An object that holds all the data needed to serialize or deserialize this instance.</param>
        /// <param name="context">Describes the source and destination of the serialized stream specified by info. </param>
        protected FKCellModelSelectNew(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            base.ButtonBarSize = new Size(20, 20);
        }

        /// <summary>
        /// Initializes a new <see cref="FKCellModelSelectNew"/> object 
        /// and stores a reference to the <see cref="GridModel"/> this cell belongs to.
        /// </summary>
        /// <param name="grid">The <see cref="GridModel"/> for this cell model.</param>	
        /// <remarks>
        /// You typically access cell models through the <see cref="GridModel.CellModels"/>
        /// property of the <see cref="GridModel"/> class.
        /// </remarks>
        public FKCellModelSelectNew(GridModel grid)
            : base(grid)
        {
            base.ButtonBarSize = new Size(20, 20);
        }

        /// <override/>
        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new FKCellRendererSelectNew(control, this);
        }


        /// <override/>
        protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds GridQueryBounds)
        {
            Size size = base.OnQueryPrefferedClientSize(g, rowIndex, colIndex, style, GridQueryBounds);
            return new Size(size.Width + 20, size.Height); // base method already consides ButtonBarSize, but let's add some more pixels here.
        }


    }

    /// <summary>
    /// Implements the renderer part for a textbox cell with FK button
    /// </summary>
    public class FKCellRendererSelectNew : GridTextBoxCellRenderer
    {
        FKCellButtonNewProperties m_btnNew;

        /// <summary>
        /// Initializes a new <see cref="GridTextBoxCellRenderer"/> object for the given GridControlBase
        /// and <see cref="GridTextBoxCellModel"/>.
        /// </summary>
        /// <param name="grid">The <see cref="GridControlBase"/> that display this cell renderer.</param>
        /// <param name="cellModel">The <see cref="GridTextBoxCellModel"/> that holds data for this cell renderer that should
        /// be shared among views.</param>
        /// <remarks>References to GridControlBase, 
        /// and GridTextBoxCellModel will be saved.</remarks>
        public FKCellRendererSelectNew(GridControlBase grid, GridTextBoxCellModel cellModel)
            : base(grid, cellModel)
        {
            m_btnNew = new FKCellButtonNewProperties(this);
            AddButton(m_btnNew);
        }

        /// <override/>
        public override void OnPrepareViewStyleInfo(GridPrepareViewStyleInfoEventArgs e)
        {
            // Here you can adjust settings that override the default settings from the grid
            //			e.Style.ShowButtons = GridShowButtons.Show;
            e.Style.Clickable = true;
            e.Style.TextAlign = GridTextAlign.Left;
            e.Style.VerticalAlignment = GridVerticalAlignment.Middle;
            e.Style.WrapText = false;
        }

        /// <override/>
        protected override Rectangle OnLayout(int rowIndex, int colIndex, GridStyleInfo style, Rectangle innerBounds, Rectangle[] buttonsBounds)
        {
            TraceUtil.TraceCurrentMethodInfo(rowIndex, colIndex, style, innerBounds, buttonsBounds);

            Rectangle rightArea = Rectangle.FromLTRB(innerBounds.Right - 20, innerBounds.Top, innerBounds.Right, innerBounds.Bottom);
            buttonsBounds[0] = new Rectangle(rightArea.X, rightArea.Y, 20, rightArea.Height);

            //To Check the button shown or not shown. 
            if (style.ShowButtons == GridShowButtons.ShowCurrentCell || style.ShowButtons == GridShowButtons.ShowCurrentCellEditing)
            {
                // For ShowCurrentCell option button show after click the cell.
                if (rowIndex == this.CurrentCell.RowIndex && colIndex == this.CurrentCell.ColIndex)
                {
                    innerBounds.Width -= 20; //For button shown
                }
            }
            else if (style.ShowButtons == GridShowButtons.ShowCurrentRow)
            {
                // For ShowCurrentCell option button show after click the cell.
                if (rowIndex == this.CurrentCell.RowIndex)
                {
                    innerBounds.Width -= 20; //For button shown
                }
            }
            else if (style.ShowButtons != GridShowButtons.Hide)
            {
                innerBounds.Width -= 20;
            }

            return innerBounds;
        }

        protected override void OnDrawCellButton(GridCellButton button, Graphics g, int rowIndex, int colIndex, bool bActive, GridStyleInfo style)
        {
            base.OnDrawCellButton(button, g, rowIndex, colIndex, bActive, style);
        }
    }

}
